package com.winhc.bigdata.spark.ng.dynamic.utils

import com.winhc.bigdata.spark.implicits.Bool._
import com.winhc.bigdata.spark.ng.dynamic.utils.DynamicDimConfiguration.DDC
import org.apache.commons.lang3.StringUtils
import org.json4s._
import org.json4s.jackson.JsonMethods._
import org.json4s.jackson.Serialization

/**
 * @author ZhangJi
 * @since 2021-07-08 10:51
 */
object DynamicFiledUtils {
  val SIMPLE_FILED_CONFIG: Map[String, Map[String, Map[String, Any]]] = DDC.filter(e => e._2.getOrElse("sample", false).asInstanceOf[Boolean]).map(e => (e._1, e._2("list_field").asInstanceOf[Map[String, Map[String, Any]]]))

  def createSimpleDynamicInfo(tn: String, data: Map[String, String], update_type: String = "insert"): (String, Map[String, Any]) = {
    val config = SIMPLE_FILED_CONFIG.getOrElse(tn, Map.empty)
    val filter_set = config.filter(e => e._2.getOrElse("when", "insert").equals(update_type)).map(e => e._2.getOrElse("from", e._1)).toSet
    (DDC(tn)("code").asInstanceOf[Map[String, String]].getOrElse(update_type, null),
      data.filterKeys(e => filter_set.contains(e)))
  }

  def distinctBy[A, B](xs: Seq[A])(f: A => B): List[A] =
    scala.reflect.internal.util.Collections.distinctBy(xs.toList)(f)

  def createDynamicInfo(tn: String, data: Map[String, String]): (String, Map[String, Any]) = {
    val lf = DDC(tn)("list_field").asInstanceOf[Map[String, Map[String, Any]]]
    val dynamicInfo = lf.map(elem => {
      val from = elem._2.getOrElse("from", Seq(elem._1)).asInstanceOf[Seq[String]]
      elem._2.get("type") match {
        case Some("array") => {
          (elem._1, distinctBy(from.flatMap(n => {
            val v = data.getOrElse(n, null)
            if (StringUtils.isEmpty(v)) {
              List.empty[JValue]
            } else {
              var array = parse(v, useBigDecimalForDouble = true).asInstanceOf[JArray]
              array = (array == null) ? new JArray(List.empty[JValue]) | array
              array.arr
            }
          }))(jo => (jo \ "name")))
        }
        case Some("object") => {
          (elem._1, from.map(e => {
            val v = data.getOrElse(e, null)
            if (StringUtils.isEmpty(v)) {
              (e.replace("_info", ""), null)
            } else
              (e.replace("_info", ""), parse(data.getOrElse(e, null)))
          }).toMap)
        }
        case _ => (elem._1, data.getOrElse(from.head, null))
      }
    })
    (DDC(tn)("code").asInstanceOf[Map[String, String]]("insert"), dynamicInfo)
  }


  def main(args: Array[String]): Unit = {
    val str =
      """
        |
        |{
        |   "rowkey": "000074b9fdcdcfc57c6087fe586a9c98",
        |   "litigant_info": null,
        |   "plaintiff_info": null,
        |   "defendant_info": null,
        |   "third_info": "",
        |   "litigant": "金良,中国建筑第二工程局有限公司,重庆市中强建筑劳务有限公司",
        |   "plaintiff": "金良",
        |   "defendant": "中国建筑第二工程局有限公司,重庆市中强建筑劳务有限公司",
        |   "case_no": "（2016）辽0103民初9478号",
        |   "case_reason": "劳务合同纠纷",
        |   "start_time": null,
        |   "court": "沈阳市沈河区人民法院",
        |   "department": "",
        |   "area": "辽宁",
        |   "assistant": null,
        |   "judge": "",
        |   "third": "",
        |   "filing_date": "2016-08-23 00:00:00",
        |   "close_date": null,
        |   "content": "原告：金良；被告：中国建筑第二工程局有限公司；被告：重庆市中强建筑劳务有限公司；",
        |   "case_status": "结",
        |   "case_type": "",
        |   "source": null,
        |   "source_url": "http://lnsfw.lnsfy.gov.cn/gateway/ssfw/ssfw_app/app/wajjbxx  c393b2ed289e598026575d5ac2fb54d5",
        |   "create_time": "2020-09-05 01:46:04",
        |   "update_time": "2020-09-05 01:46:04",
        |   "deleted": 0
        | }
        |
        |""".stripMargin
    implicit val formats: AnyRef with Formats = Serialization.formats(NoTypeHints)
    val value: Map[String, String] = parse(str).extract[Map[String, String]]

    import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
    val tuple = createDynamicInfo("company_court_register", value)
    println(tuple._2.toJson())

  }
}
